#ifndef _PW_ProWesS_H
#define _PW_ProWesS_H

/*
	ProWesS, PROGS Windowing System

	Standard (public) definitions.

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 7 January 1995
	23.12.1997 some small changes for v2
*/

#include <sms.h>
#include <err.h>

#ifndef _PF_PROFORMA_H
#include <PROforma.h>
#endif

/* general types which are visible to the ProWesS user */

typedef unsigned long PWObject;
typedef unsigned long PWType;

#define PW_OBJECT_NONE (PWObject)0

/* prototypes for public routines */
Error PWCreate(PWObject owner, PWObject *ret, PWType type, ...);
Error PWDoCreate(PWObject owner, PWObject *ret, PWType type, int *taglist);
Error PWChange(PWObject object, ...);
Error PWDoChange(PWObject object, int *taglist);
Error PWQuery(PWObject object, int what, void *ret);
Error PWActivate(PWObject object);
Error PWRemove(PWObject object);
Error PWRemoveOne(PWObject object);

/* deprecated, just for copatibility, does nothing */
Error PWAddType(void *newtype);

/* access to ProWesS thing for dynamic configuration */
#define PW_THING_NAME	"ProWesS"
#define PW_THING_CNFG	0x434e4647	/* "CNFG" */

/* TAGS, these are used to identify which parameters will be changed */
/* The tags are 32 bit numbers. However, the top 20 bits are reserved to */
/* give extra information. There are two sets of tags, general tags, which */
/* can apply to any object/type, and (type) specific tags, which only apply
/* to certain objects/types. There should be no overlap between */
/* the general and specific tags. However specific tags for different types */
/* may overlap (though this may introduce unwanted synonyms). Therefore, it */
/* is adviced that anybody who distributes new types acquires a mask for the */
/* tags, to indicate the software producer. */
/* The tag contains 3 fields : */
/* tag identifier, 8 bits, identifies length of parameters */
/* manufacturer, 12 bits, identifies instances which determined the tag number */
/*						  value : 0 for general, 1 for standard */
/* tag number, 12 bits, identifies the use of the tag, has to be non-zero */

#define PW_TAGMASK_LIST    (int)0x80000000	/* parameter is a NULL terminated list */
#define PW_TAGMASK_TAGLIST		0x40000000	/* parameter is a NULL terminated list of tags */
#define PW_TAGMASK_LOCAL		0x20000000	/* no passing to parent on query */
#define PW_TAGMASK_MULBIT		0x10000000	/* bit set when skip specific */
#define PW_TAGMASK_MULTIPLE(x)	((0x10+x)<<24)/* parameter is "x" values */
#define PW_TAGINFO_SKIP(x)		(((x)>>24)&0xf) /* extract skip from tag */
#define PW_TAGMASK_SKIP    (int)0xff000000	/* skip this tag and parameter */
#define PW_TAGMASK_PRODUCER 	0x00fff000	/* manufacturer type */
#define PW_TAGMASK_STANDARD 	0x00001000	/* mask for standard tags */

#define PW_QUERY_LOCAL(x)		( (x)|PW_TAGMASK_LOCAL )

#include <system.pwt>
#include <standard.pwt>

#endif _PW_ProWesS_H
