#ifndef _SYSLIB_THING_H
#define _SYSLIB_THING_H

/*
	Standard header file for PROGS SMS C library

	thing handling library

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 4 December 1994
*/

#include <io.h>
#include <job.h>

#define THING_MAXNAME 64			/* max length for a thingname */

typedef struct {
	int extension;
	Error (*function)();
	} ThingExtension;

#define THING_LISTSENTINEL 0,(Error(*)())NULL

typedef struct {
	char name[THING_MAXNAME];
	int version;
	char sharable;
	Error (*use_code)();
	void (*free_code)();
	void (*zap_code)();
	void (*remove_code)();
	Job owner;
	ThingExtension *list;			/* array, last extension should be NULL */
	} ThingDefinition;

typedef struct _Thing *Thing;

Error THINGCreate(ThingDefinition *def);
Error THINGCreateBlock(ThingDefinition *def);
Error THINGCreateAny(ThingDefinition *def, Thing thing);
Error THINGAppend(ThingDefinition *def);
Error THINGAppendBlock(ThingDefinition *def);
Error THINGRemove(char *name);
Error THINGUse(char *name, int extension, Thing *thing, int *version);
Error THINGUseFor(char *name, int extension, Job job, TimeOut timeout, Thing *thing, int *version);
Error THINGUseFull(char *name, int pard2, Job job, TimeOut timeout, int para2, int *retd2, Thing *thing, int *version);
Error THINGFree(char *name);
Error THINGFreeFor(char *name, Job job);
Error THINGFreeFull(char *name, Job job, int pard2, int pard3, int para1, int para2, int *retd2, int *reta2);
Error THINGZap(char *name);
Error THINGNext(char *name);
Error THINGUserNext(char *name, int *counter, Job *user);

Error THINGCall(char *name, int extension, int paramcount, ...);
Error THINGCallIn(Thing th, int extension, int paramcount, ...);

Error THINGCallBlock(char *name, int extension, void *paramblock);
Error THINGCallBlockIn(Thing th, int extension, void *paramblock);

Error THINGExec(char *name, Priority priority, Job owner, char wait,
		int stringcount, char **strings, int channelcount, Channel *channels,
		Job *jobid);

#endif _SYSLIB_THING_H
