#ifndef _SYSLIB_WIN_H
#define _SYSLIB_WIN_H

/*
	Standard header file for PROGS SMS C library

	Window handling library

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 04 January 1995
*/

#include <io.h>

typedef Channel Window; 			/* Window identifier type */

typedef struct {
	short xsiz, ysiz, xpos, ypos;	/* size and position for window */
	short xshadow, yshadow; 		/* shadow width */
	short border_colour, border_width;	/* border colour and width */
	short paper_colour, draw_colour;	/* paper colour and draw colour */
} WindowDescriptor;

typedef struct {
	short xsiz, ysiz;				/* window size, in pixels or characters */
	short xpos, ypos;				/* cursor position, idem */
} WindowInfo;

#define WINDOWMODE_2		(WindowMode)2	/* two colour (mono) mode */
#define WINDOWMODE_M4		(WindowMode)0	/* four colour mixed plane mode */
#define WINDOWMODE_M8		(WindowMode)8	/* eight colour mixed plane mode */
#define WINDOWMODE_8BIT 	(WindowMode)16	/* Aurora 8 bit colour mode */
#define WINDOWMODE_8BITAURORA (WindowMode)16/* Aurora 8 bit colour mode */
#define WINDOWMODE_16BIT	(WindowMode)33	/* Q40 16 bit colour mode */
#define WINDOWMODE_16BITQ40 (WindowMode)33	/* Q40 16 bit colour mode */
#define WINDOWMODE_16BITQPC (WindowMode)32	/* QPC/QXL 16 bit colour mode */
#define WINDOWMODE_16BITQXL (WindowMode)32	/* QPC/QXL 16 bit colour mode */
typedef char WindowMode;			/* Window Display Mode */

#define WINDOWAREA_FLAG 0x4afc
typedef struct _WindowArea {
	struct _WindowArea *next;		/* to be used by client */
	unsigned short flag;			/* flag, should be WINDOWAREA_FLAG */
	short xsiz, ysiz;				/* size of area */
	short line_increment;			/* line increment */
	WindowMode mode;				/* windowmode of saved area */
	char zero;						/* zero byte */
	char data[2];					/* start of data */
} WindowArea;

typedef struct {
	short form, adaption;
	short xsiz, ysiz;
	short xorg, yorg;
	char *rel_colour;				/* relative pointer to colour pattern */
	char *rel_mask; 				/* relative pointer to mask */
	char *rel_next; 				/* relative pointer to next sprite */
} Sprite;

extern Sprite *SPRITE_ARROW;
extern Sprite *SPRITE_HAND;
extern Sprite *SPRITE_LEFT;
extern Sprite *SPRITE_MOVE;
extern Sprite *SPRITE_SIZE;
extern Sprite *SPRITE_SLEEP;
extern Sprite *SPRITE_WAKE;
extern Sprite *SPRITE_NULL;

typedef struct {
	short xsiz, ysiz;				/* size of sub-window */
	short xpos, ypos;				/* origin of sub-window */
	char cursor, spare; 			/* is cursor independant ?? */
	short border_width, border_colour, paper_colour; /* irrelevant */
	Sprite *sprite; 				/* sprite for this sub-window */
} SubWindow;

typedef struct {
	SubWindow **list;				/* ptr to NULL terminated list (array) */
									/* of type (SubWindow *) */
	SubWindow main; 				/* definition for main window */
} SubWindowList;

#define EVENT_KEYSTROKE 	(WindowEvent)1
#define EVENT_KEYDOWN		(WindowEvent)1	/* key has been pressed down */
#define EVENT_KEYPRESS		(WindowEvent)2	/* key is depressed (autorepeat!) */
#define EVENT_KEYUP 		(WindowEvent)4	/* key has been released */
#define EVENT_MOVE			(WindowEvent)8
#define EVENT_OUT_WINDOW	(WindowEvent)16
#define EVENT_IN_WINDOW 	(WindowEvent)32
#define EVENT_TOUCH_EDGE	(WindowEvent)64
#define EVENT_WINDOW_MOVE	(WindowEvent)129	/* do not combine ! */
#define EVENT_WINDOW_SIZE	(WindowEvent)130	/* do not combine ! */
#define EVENT_WAKE			(WindowEvent)256	/* wake occured - return only */
#define EVENT_LEVEL1		(WindowEvent)0x01000000
#define EVENT_LEVEL2		(WindowEvent)0x02000000
#define EVENT_LEVEL3		(WindowEvent)0x04000000
#define EVENT_LEVEL4		(WindowEvent)0x08000000
#define EVENT_LEVEL5		(WindowEvent)0x10000000
#define EVENT_LEVEL6		(WindowEvent)0x20000000
#define EVENT_LEVEL7		(WindowEvent)0x40000000
#define EVENT_LEVEL8		(WindowEvent)0x80000000
#define EVENT_LEVELS		(WindowEvent)0xff000000
typedef unsigned long WindowEvent;	/* type for a window event */

typedef struct {
	Window id;						/* window containing pointer or -1 */
	short subwindow;				/* subwindow number or -1 */
	short xpos,ypos;				/* pointer position or -1,-1 */
	char keystroke, keypress;		/* keystroke and keypress */
	WindowEvent event;				/* event vector */
	WindowInfo sub; 				/* subwindow info */
} PointerInfo;

#define OVERWRITE_TRANSPARENT 1 	/* transpared background */
#define OVERWRITE_NORMAL 0			/* background drawn in strip colour */
#define OVERWRITE_EOR -1			/* ink eored in background */

#define BORDER_TRANSPARENT	0x80	/* used as colour for transparent border */

#define WINDOW_NONE 	(Window)-1	/* invalid window id */

#define KEYSTROKE_NONE		(char)0
#define KEYSTROKE_HIT		(char)1
#define KEYSTROKE_DO		(char)2
#define KEYSTROKE_CANCEL	(char)3
#define KEYSTROKE_HELP		(char)4
#define KEYSTROKE_MOVE		(char)5
#define KEYSTROKE_SIZE		(char)6
#define KEYSTROKE_SLEEP 	(char)7
#define KEYSTROKE_WAKE		(char)8

Error WINOpen(short xsiz, short ysiz, short xpos, short ypos, Window *id);
Error WINOpenFull(WindowDescriptor *descriptor, Window *id);
Error WINClose(Window);

Error WINGet(Window id, char *ret);
Error WINPut(Window id, char value);
#define WINPut(w,c) IOPut(w,c)
Error WINPutString(Window id, char *string);
#define WINPutString(w,s) IOPutString(w,s,NULL)

Error WINTest(Window id);
#define WINTest(w) IOTest(w)

Error WINTimeOutSet(Window id, TimeOut t);
#define WINTimeOutSet(w,t) IOTimeOutSet(w,t)
Error WINTimeOutGet(Window id, TimeOut *t);
#define WINTimeOutGet(w,t) IOTimeOutGet(w,t)
Error WINPutStringAddNewLineGet(Window id, char *status);
#define WINPutStringAddNewLineGet(w,s) IOPutStringAddNewLineGet(w,s)
Error WINPutStringAddNewLineSet(Window id, char status);
#define WINPutStringAddNewLineSet(w,s) IOPutStringAddNewLineSet(w,s)

Error WINWindow(Window id, short xsiz, short ysiz, short xpos, short ypos);
Error WINBorder(Window id, short width, short colour);
Error WINRedefine(Window id, short width, short colour,
				  short xsiz, short ysiz, short xpos, short ypos);
Error WINPaper(Window id, short colour);
Error WINStrip(Window id, short colour);
Error WINInk(Window id, short colour);

Error WINClear(Window id);
Error WINClearTop(Window id);
Error WINClearBottom(Window id);
Error WINClearLine(Window id);
Error WINClearRight(Window id);

Error WINCursorEnable(Window id);
Error WINCursorDisable(Window id);
Error WINPendingNewLine(Window id);
Error WINNewLine(Window id);
Error WINColumnPrev(Window id);
Error WINColumnNext(Window id);
Error WINRowPrev(Window id);
Error WINRowNext(Window id);
Error WINCursorSize(Window id, short xsiz, short ysiz);
Error WINAtCursor(Window id, short xpos, short ypos);
Error WINAtColumn(Window id, short ypos);
Error WINAtPixel(Window id, short xpos, short ypos);

Error WINOverWrite(Window id, short status);
Error WINUnderLine(Window id, short status);

Error WINScroll(Window id, short distance);
Error WINScrollTop(Window id, short distance);
Error WINScrollBottom(Window id, short distance);
Error WINPan(Window id, short distance);
Error WINPanLine(Window id, short distance);
Error WINPanRight(Window id, short distance);

Error WINInfoCursor(Window id, WindowInfo *ret);
Error WINInfoPixel(Window id, WindowInfo *ret);

Error WINLimits(Window id, WindowInfo *ret);

Error WINBlock(Window id, short colour,
			   short xsiz, short ysiz, short xpos, short ypos);
Error WINSprite(Window id, short xpos, short ypos, Sprite *sprite);

Error WINAreaAllocate(short xsiz, short ysiz, WindowMode mode,
					  WindowArea **area);
Error WINAreaRelease(WindowArea *area);
Error WINAreaSave(Window id, WindowInfo *info, short xpos, short ypos,
				  WindowArea *area);
Error WINAreaRestore(Window id, WindowInfo *info, short xpos, short ypos,
				  WindowArea *area);
Error WINMode(WindowMode *mode);

Error WINPointerRead(Window id, TimeOut timeout, WindowEvent event,
					 short *xpos, short *ypos, PointerInfo *pinfo);
Error WINOutline(Window id, short xshadow, short yshadow,
				 short xsiz, short ysiz, short xpos, short ypos);
Error WINMove(Window id, short xpos, short ypos);
Error WINSubWindows(Window id, SubWindowList *subwindows);

Error WINPointerPositionSet(Window id, short xpos, short ypos);
Error WINPointerPositionGet(Window id, short *xpos, short *ypos);
Error WINPointerLimits(Window id, short xorg, short yorg, short xend, short yend);

Error WINEventSend(Job job, WindowEvent event);
Error WINEventWait(TimeOut timeout, WindowEvent *event);

#endif _SYSLIB_WIN_H
